#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
# try:
#     from osgeo.osr import SpatialReference, CoordinateTransformation
#     have_gdal = True
# except ImportError:
#     have_gdal = False

# import interfaces
from onf_website.component.location.interfaces import ILocationInfo, ILocationTarget, LOCATION_INFO_KEY
from pyams_content_es.interfaces import IDocumentIndexInfo
from pyams_form.interfaces.form import IFormContextPermissionChecker
from zope.annotation.interfaces import IAnnotations
from zope.intid.interfaces import IIntIds

# import packages
from persistent import Persistent
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyramid.threadlocal import get_current_registry
from zope.container.contained import Contained
from zope.interface import implementer
from zope.lifecycleevent import ObjectCreatedEvent
from zope.location import locate
from zope.schema.fieldproperty import FieldProperty


@implementer(ILocationInfo)
class LocationInfo(Persistent, Contained):
    """Location info persistent class"""

    _inherit = FieldProperty(ILocationInfo['inherit'])
    _countries = FieldProperty(ILocationInfo['countries'])
    _departments = FieldProperty(ILocationInfo['departments'])
    _structures = FieldProperty(ILocationInfo['structures'])
    _address = FieldProperty(ILocationInfo['address'])
    _postal_code = FieldProperty(ILocationInfo['postal_code'])
    _city = FieldProperty(ILocationInfo['city'])
    _situation = FieldProperty(ILocationInfo['situation'])
    _natural_region = FieldProperty(ILocationInfo['natural_region'])
    _forests = FieldProperty(ILocationInfo['forests'])
    _forests_label = FieldProperty(ILocationInfo['forests_label'])
    _gps_location = FieldProperty(ILocationInfo['gps_location'])

    @property
    def parent(self):
        return get_parent(self.__parent__, ILocationTarget, allow_context=False)

    @property
    def can_inherit(self):
        return ILocationTarget.providedBy(self.parent)

    @property
    def inherit(self):
        return self._inherit if self.can_inherit else False

    @inherit.setter
    def inherit(self, value):
        if self.can_inherit:
            self._inherit = value

    @property
    def no_inherit(self):
        return not bool(self.inherit)

    @no_inherit.setter
    def no_inherit(self, value):
        self.inherit = not bool(value)

    @property
    def countries(self):
        return ILocationInfo(self.parent).countries if self.inherit else self._countries

    @countries.setter
    def countries(self, value):
        if not (self.can_inherit and self.inherit):
            self._countries = value

    @property
    def countries_index(self):
        intids = get_utility(IIntIds)
        return [intids.register(value) for value in self.countries or ()]

    @property
    def departments(self):
        return ILocationInfo(self.parent).departments if self.inherit else self._departments

    @departments.setter
    def departments(self, value):
        if not (self.can_inherit and self.inherit):
            self._departments = value

    @property
    def departments_index(self):
        intids = get_utility(IIntIds)
        return [intids.register(value) for value in self.departments or ()]

    @property
    def structures(self):
        return ILocationInfo(self.parent).structures if self.inherit else self._structures

    @structures.setter
    def structures(self, value):
        if not (self.can_inherit and self.inherit):
            self._structures = value

    @property
    def address(self):
        return ILocationInfo(self.parent).address if self.inherit else self._address

    @address.setter
    def address(self, value):
        if not (self.can_inherit and self.inherit):
            self._address = value

    @property
    def postal_code(self):
        return ILocationInfo(self.parent).postal_code if self.inherit else self._postal_code

    @postal_code.setter
    def postal_code(self, value):
        if not (self.can_inherit and self.inherit):
            self._postal_code = value

    @property
    def city(self):
        return ILocationInfo(self.parent).city if self.inherit else self._city

    @city.setter
    def city(self, value):
        if not (self.can_inherit and self.inherit):
            self._city = value

    @property
    def situation(self):
        return ILocationInfo(self.parent).situation if self.inherit else self._situation

    @situation.setter
    def situation(self, value):
        if not (self.can_inherit and self.inherit):
            self._situation = value

    @property
    def natural_region(self):
        return ILocationInfo(self.parent).natural_region if self.inherit else self._natural_region

    @natural_region.setter
    def natural_region(self, value):
        if not (self.can_inherit and self.inherit):
            self._natural_region = value

    @property
    def forests(self):
        return ILocationInfo(self.parent).forests if self.inherit else self._forests

    @forests.setter
    def forests(self, value):
        if not (self.can_inherit and self.inherit):
            self._forests = value

    @property
    def forests_label(self):
        return ILocationInfo(self.parent).forests_label if self.inherit else self._forests_label

    @forests_label.setter
    def forests_label(self, value):
        if not (self.can_inherit and self.inherit):
            self._forests_label = value

    @property
    def gps_location(self):
        return ILocationInfo(self.parent).gps_distance if self.inherit else self._gps_location

    @gps_location.setter
    def gps_location(self, value):
        if not (self.can_inherit and self.inherit):
            self._gps_location = value


@adapter_config(context=ILocationTarget, provides=ILocationInfo)
def LocationInfoFactory(target):
    """Location info factory"""
    annotations = IAnnotations(target)
    location = annotations.get(LOCATION_INFO_KEY)
    if location is None:
        location = annotations[LOCATION_INFO_KEY] = LocationInfo()
        get_current_registry().notify(ObjectCreatedEvent(location))
        locate(location, target, '++location++')
    return location


@adapter_config(context=ILocationInfo, provides=IFormContextPermissionChecker)
class LocationInfoPermissionChecker(ContextAdapter):
    """Location info checker"""

    @property
    def edit_permission(self):
        content = get_parent(self.context, ILocationTarget)
        return IFormContextPermissionChecker(content).edit_permission


@adapter_config(name='location', context=ILocationTarget, provides=IDocumentIndexInfo)
def LocationTargetIndexInfo(content):
    """Location index info"""
    location = ILocationInfo(content)
    values = {'city': location.city}
    if location.gps_location:
        values['coords'] = location.gps_location.wgs_coordinates
    return {'location': values}
