#
# Copyright (c) 2008-2017 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from persistent import Persistent
try:
    from osgeo.osr import SpatialReference, CoordinateTransformation
    have_gdal = True
except ImportError:
    have_gdal = False

# import interfaces
from onf_website.component.location.interfaces import IViewLocationSettings, VIEW_LOCATION_SETTINGS_KEY, ILocationInfo
from pyams_content.shared.view.interfaces import IWfView, IViewExtension
from zope.annotation.interfaces import IAnnotations

# import packages
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyramid.threadlocal import get_current_registry
from zope.container.contained import Contained
from zope.interface import implementer
from zope.lifecycleevent import ObjectCreatedEvent
from zope.location import locate
from zope.schema.fieldproperty import FieldProperty


@implementer(IViewLocationSettings)
class ViewLocationSettings(Persistent, Contained):
    """View location settings"""

    select_context_countries = FieldProperty(IViewLocationSettings['select_context_countries'])
    countries = FieldProperty(IViewLocationSettings['countries'])
    select_context_departments = FieldProperty(IViewLocationSettings['select_context_departments'])
    departments = FieldProperty(IViewLocationSettings['departments'])
    select_context_structures = FieldProperty(IViewLocationSettings['select_context_structures'])
    structures = FieldProperty(IViewLocationSettings['structures'])
    select_context_postal_code = FieldProperty(IViewLocationSettings['select_context_postal_code'])
    postal_code = FieldProperty(IViewLocationSettings['postal_code'])
    select_context_city = FieldProperty(IViewLocationSettings['select_context_city'])
    city = FieldProperty(IViewLocationSettings['city'])
    select_context_forests = FieldProperty(IViewLocationSettings['select_context_forests'])
    forests = FieldProperty(IViewLocationSettings['forests'])
    select_context_gps_location = FieldProperty(IViewLocationSettings['select_context_gps_location'])
    gps_location = FieldProperty(IViewLocationSettings['gps_location'])
    gps_distance = FieldProperty(IViewLocationSettings['gps_distance'])
    gps_area = FieldProperty(IViewLocationSettings['gps_area'])

    def get_countries(self, context):
        countries = set()
        if self.select_context_countries:
            location = ILocationInfo(context, None)
            if location is not None:
                countries |= location.countries or set()
        if self.countries:
            countries |= self.countries
        return countries

    def get_departments(self, context):
        departments = set()
        if self.select_context_departments:
            location = ILocationInfo(context, None)
            if location is not None:
                departments |= location.departments or set()
        if self.departments:
            departments |= self.departments
        return departments

    def get_structures(self, context):
        structures = set()
        if self.select_context_structures:
            location = ILocationInfo(context, None)
            if location is not None:
                structures |= location.structures or set()
        if self.structures:
            structures |= self.structures
        return structures

    def get_postal_codes(self, context):
        postal_codes = set()
        if self.select_context_postal_code:
            location = ILocationInfo(context, None)
            if location is not None and location.postal_code:
                postal_codes.add(location.postal_code)
        if self.postal_code:
            postal_codes.add(self.postal_code)
        return postal_codes

    def get_cities(self, context):
        cities = set()
        if self.select_context_city:
            location = ILocationInfo(context, None)
            if location is not None and location.city:
                cities.add(location.city)
        if self.city:
            cities.add(self.city)
        return cities

    def get_forests(self, context):
        forests = set()
        if self.select_context_forests:
            location = ILocationInfo(context, None)
            if location is not None:
                forests |= location.forests or set()
        if self.forests:
            forests.add(self.forests)
        return forests

    def get_gps_location(self, context):
        if self.select_context_gps_location:
            location = ILocationInfo(context, None)
            if location is not None:
                coords = location.gps_location.wgs_coordinates
                if coords and coords[0] and coords[1]:
                    return {'location': 'point',
                            'position': coords,
                            'distance': self.gps_location.gps_distance}
        else:
            if self.gps_location:
                coords = self.gps_location.wgs_coordinates
                if coords and coords[0] and coords[1]:
                    return {'location': 'point',
                            'position': coords,
                            'distance': self.gps_location.gps_distance}
            elif self.gps_area:
                coords = self.gps_area.wgs_coordinates
                if coords and coords[0] and coords[1] and coords[2] and coords[3]:
                    return {'location': 'rect',
                            'area': coords}


@adapter_config(context=IWfView, provides=IViewLocationSettings)
def ViewLocationSettingsFactory(view):
    """View location settings factory"""
    annotations = IAnnotations(view)
    settings = annotations.get(VIEW_LOCATION_SETTINGS_KEY)
    if settings is None:
        settings = annotations[VIEW_LOCATION_SETTINGS_KEY] = ViewLocationSettings()
        get_current_registry().notify(ObjectCreatedEvent(settings))
        locate(settings, view, '++view:location++')
    return settings


@adapter_config(name='location', context=IWfView, provides=IViewExtension)
class ViewLocationExtension(ContextAdapter):
    """View location extension"""

    def get_params(self, context):
        pass

    weight = 50

    def filter(self, context, items):
        return items
